@@ -1,58 +1,79 @@
+;;;; allocation VOPs for Mips
+
+;;;; This software is part of the SBCL system. See the README file for
+;;;; more information.
+;;;;
+;;;; This software is derived from the CMU CL system, which was
+;;;; written at Carnegie Mellon University and released into the
+;;;; public domain. The software is in the public domain and is
+;;;; provided with absolutely no warranty. See the COPYING and CREDITS
+;;;; files for more information.
+
 (in-package "SB!VM")
 
 
 ;;;; LIST and LIST*
+(defoptimizer (list stack-allocate-result) ((&rest args))
+  (not (null args)))
+(defoptimizer (list* stack-allocate-result) ((&rest args))
+  (not (null (rest args))))
 
 (define-vop (list-or-list*)
   (:args (things :more t))
   (:temporary (:scs (descriptor-reg) :type list) ptr)
   (:temporary (:scs (descriptor-reg)) temp)
   (:temporary (:scs (descriptor-reg) :type list :to (:result 0) :target result)
-	      res)
+              res)
   (:temporary (:sc non-descriptor-reg :offset nl4-offset) pa-flag)
   (:info num)
   (:results (result :scs (descriptor-reg)))
   (:variant-vars star)
   (:policy :safe)
+  (:node-var node)
   (:generator 0
     (cond ((zerop num)
-	   (move result null-tn))
-	  ((and star (= num 1))
-	   (move result (tn-ref-tn things)))
-	  (t
-	   (macrolet
-	       ((store-car (tn list &optional (slot cons-car-slot))
-		  `(let ((reg
-			  (sc-case ,tn
-			    ((any-reg descriptor-reg) ,tn)
-			    (zero zero-tn)
-			    (null null-tn)
-			    (control-stack
-			     (load-stack-tn temp ,tn)
-			     temp))))
-		     (storew reg ,list ,slot list-pointer-lowtag))))
-	     (let ((cons-cells (if star (1- num) num)))
-	       (pseudo-atomic (pa-flag
-			       :extra (* (pad-data-block cons-size)
-					 cons-cells))
-		 (inst or res alloc-tn list-pointer-lowtag)
-		 (move ptr res)
-		 (dotimes (i (1- cons-cells))
-		   (store-car (tn-ref-tn things) ptr)
-		   (setf things (tn-ref-across things))
-		   (inst addu ptr ptr (pad-data-block cons-size))
-		   (storew ptr ptr
-			   (- cons-cdr-slot cons-size)
-			   list-pointer-lowtag))
-		 (store-car (tn-ref-tn things) ptr)
-		 (cond (star
-			(setf things (tn-ref-across things))
-			(store-car (tn-ref-tn things) ptr cons-cdr-slot))
-		       (t
-			(storew null-tn ptr
-				cons-cdr-slot list-pointer-lowtag)))
-		 (aver (null (tn-ref-across things)))
-		 (move result res))))))))
+           (move result null-tn))
+          ((and star (= num 1))
+           (move result (tn-ref-tn things)))
+          (t
+           (macrolet
+               ((store-car (tn list &optional (slot cons-car-slot))
+                  `(let ((reg
+                          (sc-case ,tn
+                            ((any-reg descriptor-reg zero null)
+                             ,tn)
+                            (control-stack
+                             (load-stack-tn temp ,tn)
+                             temp))))
+                     (storew reg ,list ,slot list-pointer-lowtag))))
+             (let* ((dx-p (node-stack-allocate-p node))
+                    (cons-cells (if star (1- num) num))
+                    (alloc (* (pad-data-block cons-size) cons-cells)))
+               (pseudo-atomic (pa-flag :extra (if dx-p 0 alloc))
+                 (when dx-p
+                   (align-csp res))
+                 (inst srl res (if dx-p csp-tn alloc-tn) n-lowtag-bits)
+                 (inst sll res n-lowtag-bits)
+                 (inst or res list-pointer-lowtag)
+                 (when dx-p
+                   (inst addu csp-tn alloc))
+                 (move ptr res)
+                 (dotimes (i (1- cons-cells))
+                   (store-car (tn-ref-tn things) ptr)
+                   (setf things (tn-ref-across things))
+                   (inst addu ptr ptr (pad-data-block cons-size))
+                   (storew ptr ptr
+                           (- cons-cdr-slot cons-size)
+                           list-pointer-lowtag))
+                 (store-car (tn-ref-tn things) ptr)
+                 (cond (star
+                        (setf things (tn-ref-across things))
+                        (store-car (tn-ref-tn things) ptr cons-cdr-slot))
+                       (t
+                        (storew null-tn ptr
+                                cons-cdr-slot list-pointer-lowtag)))
+                 (aver (null (tn-ref-across things)))
+                 (move result res))))))))
 
 (define-vop (list list-or-list*)
   (:variant nil))
@@ -65,7 +86,7 @@
 
 (define-vop (allocate-code-object)
   (:args (boxed-arg :scs (any-reg))
-	 (unboxed-arg :scs (any-reg)))
+         (unboxed-arg :scs (any-reg)))
   (:results (result :scs (descriptor-reg)))
   (:temporary (:scs (non-descriptor-reg)) ndescr)
   (:temporary (:scs (any-reg) :from (:argument 0)) boxed)
@@ -74,23 +95,22 @@
   (:generator 100
     (inst li ndescr (lognot lowtag-mask))
     (inst addu boxed boxed-arg
-	  (fixnumize (1+ code-trace-table-offset-slot)))
+          (fixnumize (1+ code-trace-table-offset-slot)))
     (inst and boxed ndescr)
     (inst srl unboxed unboxed-arg word-shift)
     (inst addu unboxed unboxed lowtag-mask)
     (inst and unboxed ndescr)
     (inst sll ndescr boxed (- n-widetag-bits word-shift))
     (inst or ndescr code-header-widetag)
-    
+
     (pseudo-atomic (pa-flag)
       (inst or result alloc-tn other-pointer-lowtag)
+      (inst addu alloc-tn boxed)
       (storew ndescr result 0 other-pointer-lowtag)
       (storew unboxed result code-code-size-slot other-pointer-lowtag)
+      (inst addu alloc-tn unboxed)
       (storew null-tn result code-entry-points-slot other-pointer-lowtag)
-      (inst addu alloc-tn boxed)
-      (inst addu alloc-tn unboxed))
-
-    (storew null-tn result code-debug-info-slot other-pointer-lowtag)))
+      (storew null-tn result code-debug-info-slot other-pointer-lowtag))))
 
 (define-vop (make-fdefn)
   (:policy :fast-safe)
@@ -101,28 +121,37 @@
   (:results (result :scs (descriptor-reg) :from :argument))
   (:generator 37
     (with-fixed-allocation (result pa-flag temp fdefn-widetag fdefn-size)
+      (inst li temp (make-fixup "undefined_tramp" :foreign))
       (storew name result fdefn-name-slot other-pointer-lowtag)
       (storew null-tn result fdefn-fun-slot other-pointer-lowtag)
-      (inst li temp (make-fixup "undefined_tramp" :foreign))
       (storew temp result fdefn-raw-addr-slot other-pointer-lowtag))))
 
 (define-vop (make-closure)
   (:args (function :to :save :scs (descriptor-reg)))
   (:info length stack-allocate-p)
-  (:ignore stack-allocate-p)
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:temporary (:sc non-descriptor-reg :offset nl4-offset) pa-flag)
   (:results (result :scs (descriptor-reg)))
   (:generator 10
-    (let ((size (+ length closure-info-offset)))
-      (inst li temp (logior (ash (1- size) n-widetag-bits) closure-header-widetag))
-      (pseudo-atomic (pa-flag :extra (pad-data-block size))
-	(inst or result alloc-tn fun-pointer-lowtag)
-	(storew temp result 0 fun-pointer-lowtag))
+    (let* ((size (+ length closure-info-offset))
+           (alloc-size (pad-data-block size)))
+      (pseudo-atomic (pa-flag :extra (if stack-allocate-p 0 alloc-size))
+        (cond (stack-allocate-p
+               (align-csp result)
+               (inst srl result csp-tn n-lowtag-bits)
+               (inst addu csp-tn alloc-size))
+              (t
+               (inst srl result alloc-tn n-lowtag-bits)))
+        (inst sll result n-lowtag-bits)
+        (inst or result fun-pointer-lowtag)
+        (inst li temp (logior (ash (1- size) n-widetag-bits)
+                              closure-header-widetag))
+        (storew temp result 0 fun-pointer-lowtag))
+      (storew result result closure-self-slot fun-pointer-lowtag)
       (storew function result closure-fun-slot fun-pointer-lowtag))))
 
 ;;; The compiler likes to be able to directly make value cells.
-;;; 
+;;;
 (define-vop (make-value-cell)
   (:args (value :to :save :scs (descriptor-reg any-reg null zero)))
   (:temporary (:scs (non-descriptor-reg)) temp)
@@ -152,8 +181,8 @@
     (pseudo-atomic (pa-flag :extra (pad-data-block words))
       (inst or result alloc-tn lowtag)
       (when type
-	(inst li temp (logior (ash (1- words) n-widetag-bits) type))
-	(storew temp result 0 lowtag)))))
+        (inst li temp (logior (ash (1- words) n-widetag-bits) type))
+        (storew temp result 0 lowtag)))))
 
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
