@@ -309,8 +309,7 @@
 
 (defmethod ensure-class-using-class ((class null) name &rest args &key)
   (multiple-value-bind (meta initargs)
-      (ensure-class-values class args)
-    (set-class-type-translation (class-prototype meta) name)
+      (frob-ensure-class-args args)
     (setf class (apply #'make-instance meta :name name initargs))
     (without-package-locks
       (setf (find-class name) class))
@@ -319,7 +318,7 @@
 
 (defmethod ensure-class-using-class ((class pcl-class) name &rest args &key)
   (multiple-value-bind (meta initargs)
-      (ensure-class-values class args)
+      (frob-ensure-class-args args)
     (unless (eq (class-of class) meta)
       (apply #'change-class class meta initargs))
     (apply #'reinitialize-instance class initargs)
@@ -328,35 +327,29 @@
     (set-class-type-translation class name)
     class))
 
-(defun fix-super (s)
-  (cond ((classp s) s)
-        ((not (legal-class-name-p s))
-         (error "~S is not a class or a legal class name." s))
-        (t
-         (or (find-class s nil)
-             (make-instance 'forward-referenced-class
-                            :name s)))))
-
-(defun ensure-class-values (class initargs)
+(defun frob-ensure-class-args (args)
   (let (metaclass metaclassp reversed-plist)
-    (doplist (key val) initargs
-      (cond ((eq key :metaclass)
-             (setf metaclass val
-                   metaclassp key))
-            (t
-             (when (eq key :direct-superclasses)
-               (setf val (mapcar #'fix-super val)))
-             (setf reversed-plist (list* val key reversed-plist)))))
-    (values (cond (metaclassp
-                   (if (classp metaclass)
-                       metaclass
-                       (find-class metaclass)))
-                  ((or (null class) (forward-referenced-class-p class))
-                   *the-class-standard-class*)
-                  (t
-                   (class-of class)))
-            (nreverse reversed-plist))))
-
+    (flet ((frob-superclass (s)
+             (cond
+               ((classp s) s)
+               ((legal-class-name-p s)
+                (or (find-class s nil)
+                    (ensure-class s :metaclass 'forward-referenced-class)))
+               (t (error "Not a class or a legal class name: ~S." s)))))
+      (doplist (key val) args
+        (cond ((eq key :metaclass)
+               (unless metaclassp
+                 (setf metaclass val metaclassp key)))
+              (t
+               (when (eq key :direct-superclasses)
+                 (setf val (mapcar #'frob-superclass val)))
+               (setf reversed-plist (list* val key reversed-plist)))))
+      (values (cond (metaclassp
+                     (if (classp metaclass)
+                         metaclass
+                         (find-class metaclass)))
+                    (t *the-class-standard-class*))
+              (nreverse reversed-plist)))))
 
 (defmethod shared-initialize :after
     ((class std-class) slot-names &key
@@ -410,27 +403,10 @@
                     (push old collect)))))
           (nreverse collect)))
   (add-direct-subclasses class direct-superclasses)
-  (update-class class nil)
-  (do* ((slots (slot-value class 'slots) (cdr slots))
-        (dupes nil))
-       ((null slots) (when dupes
-                       (style-warn
-                        ;; FIXME: the indentation request ("~4I")
-                        ;; below appears not to do anything.  Finding
-                        ;; out why would be nice.  -- CSR, 2003-04-24
-                        "~@<slot names with the same SYMBOL-NAME but ~
-                         different SYMBOL-PACKAGE (possible package problem) ~
-                         for class ~S:~@:_~4I~<~@{~S~^~:@_~}~:>~@:>"
-                        class
-                        dupes)))
-    (let* ((slot (car slots))
-           (oslots (remove (slot-definition-name slot) (cdr slots)
-                           :test #'string/= :key #'slot-definition-name)))
-      (when oslots
-        (pushnew (cons (slot-definition-name slot)
-                       (mapcar #'slot-definition-name oslots))
-                 dupes
-                 :test #'string= :key #'car))))
+  (if (class-finalized-p class)
+      ;; required by AMOP, "Reinitialization of Class Metaobjects"
+      (finalize-inheritance class)
+      (update-class class nil))
   (add-slot-accessors class direct-slots)
   (make-preliminary-layout class))
 
@@ -450,21 +426,12 @@
     (without-package-locks
      (unless (class-finalized-p class)
        (let ((name (class-name class)))
-         (setf (find-class name) class)
          ;; KLUDGE: This is fairly horrible.  We need to make a
          ;; full-fledged CLASSOID here, not just tell the compiler that
          ;; some class is forthcoming, because there are legitimate
          ;; questions one can ask of the type system, implemented in
          ;; terms of CLASSOIDs, involving forward-referenced classes. So.
-         (when (and (eq *boot-state* 'complete)
-                    (null (find-classoid name nil)))
-           (setf (find-classoid name)
-                 (make-standard-classoid :name name)))
-         (set-class-type-translation class name)
-         (let ((layout (make-wrapper 0 class))
-               (classoid (find-classoid name)))
-           (setf (layout-classoid layout) classoid)
-           (setf (classoid-pcl-class classoid) class)
+         (let ((layout (make-wrapper 0 class)))
            (setf (slot-value class 'wrapper) layout)
            (let ((cpl (compute-preliminary-cpl class)))
              (setf (layout-inherits layout)
@@ -472,8 +439,8 @@
                     (map 'simple-vector #'class-wrapper
                          (reverse (rest cpl))))))
            (register-layout layout :invalidate t)
-           (setf (classoid-layout classoid) layout)
-           (mapc #'make-preliminary-layout (class-direct-subclasses class))))))))
+           (set-class-type-translation class (layout-classoid layout)))))
+     (mapc #'make-preliminary-layout (class-direct-subclasses class)))))
 
 
 (defmethod shared-initialize :before ((class class) slot-names &key name)
@@ -731,17 +698,13 @@
 (defun fix-slot-accessors (class dslotds add/remove)
   (flet ((fix (gfspec name r/w)
            (let ((gf (cond ((eq add/remove 'add)
-                            (if (fboundp gfspec)
-                                (without-package-locks
-                                  (ensure-generic-function gfspec))
+                            (or (find-generic-function gfspec nil)
                                 (ensure-generic-function
                                  gfspec :lambda-list (case r/w
                                                        (r '(object))
                                                        (w '(new-value object))))))
-                           ((generic-function-p (and (fboundp gfspec)
-                                                     (fdefinition gfspec)))
-                            (without-package-locks
-                              (ensure-generic-function gfspec))))))
+                           (t
+                            (find-generic-function gfspec nil)))))
              (when gf
                (case r/w
                  (r (if (eq add/remove 'add)
@@ -784,35 +747,17 @@
 ;;; This is called by :after shared-initialize whenever a class is initialized
 ;;; or reinitialized. The class may or may not be finalized.
 (defun update-class (class finalizep)
-  ;; Comment from Gerd Moellmann:
-  ;;
-  ;; Note that we can't simply delay the finalization when CLASS has
-  ;; no forward referenced superclasses because that causes bootstrap
-  ;; problems.
   (without-package-locks
-   (when (and (not finalizep)
-              (not (class-finalized-p class))
-              (not (class-has-a-forward-referenced-superclass-p class)))
-     (finalize-inheritance class)
-     (dolist (sub (class-direct-subclasses class))
-       (update-class sub nil))
-     (return-from update-class))
-   (when (or finalizep (class-finalized-p class)
-             (not (class-has-a-forward-referenced-superclass-p class)))
-     (setf (find-class (class-name class)) class)
-     (update-cpl class (compute-class-precedence-list class))
-     ;; This invocation of UPDATE-SLOTS, in practice, finalizes the
-     ;; class.  The hoops above are to ensure that FINALIZE-INHERITANCE
-     ;; is called at finalization, so that MOP programmers can hook
-     ;; into the system as described in "Class Finalization Protocol"
-     ;; (section 5.5.2 of AMOP).
-     (update-slots class (compute-slots class))
-     (update-gfs-of-class class)
-     (update-initargs class (compute-default-initargs class))
-     (update-ctors 'finalize-inheritance :class class))
-   (unless finalizep
-     (dolist (sub (class-direct-subclasses class))
-       (update-class sub nil)))))
+    (when (or finalizep (class-finalized-p class))
+      (update-cpl class (compute-class-precedence-list class))
+      ;; This invocation of UPDATE-SLOTS, in practice, finalizes the
+      ;; class.
+      (update-slots class (compute-slots class))
+      (update-gfs-of-class class)
+      (update-initargs class (compute-default-initargs class))
+      (update-ctors 'finalize-inheritance :class class))
+    (dolist (sub (class-direct-subclasses class))
+      (update-class sub nil))))
 
 (define-condition cpl-protocol-violation (reference-condition error)
   ((class :initarg :class :reader cpl-protocol-violation-class)
@@ -909,7 +854,25 @@
               (wrapper-instance-slots-layout nwrapper) nlayout
               (wrapper-class-slots nwrapper) nwrapper-class-slots
               (wrapper-no-of-instance-slots nwrapper) nslots
-              wrapper nwrapper))
+              wrapper nwrapper)
+        (do* ((slots (slot-value class 'slots) (cdr slots))
+              (dupes nil))
+             ((null slots)
+              (when dupes
+                (style-warn
+                 "~@<slot names with the same SYMBOL-NAME but ~
+                  different SYMBOL-PACKAGE (possible package problem) ~
+                  for class ~S:~4I~@:_~<~@{~S~^~:@_~}~:>~@:>"
+                  class dupes)))
+          (let* ((slot (car slots))
+                 (oslots (remove (slot-definition-name slot) (cdr slots)
+                                 :test #'string/=
+                                 :key #'slot-definition-name)))
+            (when oslots
+              (pushnew (cons (slot-definition-name slot)
+                             (mapcar #'slot-definition-name oslots))
+                       dupes
+                       :test #'string= :key #'car)))))
       (setf (slot-value class 'finalized-p) t)
       (unless (eq owrapper nwrapper)
         (update-pv-table-cache-info class)
@@ -917,11 +880,12 @@
 
 (defun compute-class-slots (eslotds)
   (let (collect)
-    (dolist (eslotd eslotds)
-      (push (assoc (slot-definition-name eslotd)
-                   (class-slot-cells (slot-definition-class eslotd)))
-            collect))
-    (nreverse collect)))
+    (dolist (eslotd eslotds (nreverse collect))
+      (let ((cell (assoc (slot-definition-name eslotd)
+                         (class-slot-cells
+                          (slot-definition-allocation-class eslotd)))))
+        (aver cell)
+        (push cell collect)))))
 
 (defun update-gfs-of-class (class)
   (when (and (class-finalized-p class)
@@ -1142,7 +1106,9 @@
                              (list class)
                              (make-reader-method-function class slot-name)
                              "automatically generated reader method"
-                             slot-name)))
+                             :slot-name slot-name
+                             :object-class class
+                             :method-class-function #'reader-method-class)))
 
 (defmethod writer-method-class ((class slot-class) direct-slot &rest initargs)
   (declare (ignore direct-slot initargs))
@@ -1156,11 +1122,14 @@
                              (list *the-class-t* class)
                              (make-writer-method-function class slot-name)
                              "automatically generated writer method"
-                             slot-name)))
+                             :slot-name slot-name
+                             :object-class class
+                             :method-class-function #'writer-method-class)))
 
 (defmethod add-boundp-method ((class slot-class) generic-function slot-name)
   (add-method generic-function
-              (make-a-method 'standard-boundp-method
+              (make-a-method (constantly (find-class 'standard-boundp-method))
+                             class
                              ()
                              (list (or (class-name class) 'object))
                              (list class)
@@ -1275,15 +1244,19 @@
   (let* ((owrapper (class-wrapper class))
          (nwrapper (make-wrapper (wrapper-no-of-instance-slots owrapper)
                                  class)))
-      (setf (wrapper-instance-slots-layout nwrapper)
-            (wrapper-instance-slots-layout owrapper))
-      (setf (wrapper-class-slots nwrapper)
-            (wrapper-class-slots owrapper))
-      (with-pcl-lock
+    (unless (class-finalized-p class)
+      (if (class-has-a-forward-referenced-superclass-p class)
+          (return-from make-instances-obsolete class)
+          (update-cpl class (compute-class-precedence-list class))))
+    (setf (wrapper-instance-slots-layout nwrapper)
+          (wrapper-instance-slots-layout owrapper))
+    (setf (wrapper-class-slots nwrapper)
+          (wrapper-class-slots owrapper))
+    (with-pcl-lock
         (update-lisp-class-layout class nwrapper)
-        (setf (slot-value class 'wrapper) nwrapper)
-        (invalidate-wrapper owrapper :obsolete nwrapper)
-        class)))
+      (setf (slot-value class 'wrapper) nwrapper)
+      (invalidate-wrapper owrapper :obsolete nwrapper)
+      class)))
 
 (defmethod make-instances-obsolete ((class symbol))
   (make-instances-obsolete (find-class class))
@@ -1356,15 +1329,6 @@
         ;;  --    --> local     add slot
         ;;  --    --> shared    --
 
-        ;; Collect class slots from inherited wrappers. Needed for
-        ;; shared -> local transfers of inherited slots.
-        (let ((inherited (layout-inherits owrapper)))
-          (loop for i from (1- (length inherited)) downto 0
-                for layout = (aref inherited i)
-                when (typep layout 'wrapper)
-                do (dolist (slot (wrapper-class-slots layout))
-                     (pushnew slot oclass-slots :key #'car))))
-
         ;; Go through all the old local slots.
         (let ((opos 0))
           (dolist (name olayout)
@@ -1438,6 +1402,8 @@
 
 (defmethod change-class ((instance standard-object) (new-class standard-class)
                          &rest initargs)
+  (unless (class-finalized-p new-class)
+    (finalize-inheritance new-class))
   (let ((cpl (class-precedence-list new-class)))
     (dolist (class cpl)
       (macrolet
