@@ -22,7 +22,7 @@
          ,@body))))
 
 (do-external-formats (xf)
-  (with-open-file (s "/dev/null" :direction :input :external-format xf)
+  (with-open-file (s #-win32 "/dev/null" #+win32 "nul" :direction :input :external-format xf)
     (assert (eq (read-char s nil s) s))))
 
 ;;; Test standard character read-write equivalency over all external formats.
@@ -258,8 +258,7 @@
 
 ;;; See sbcl-devel "Subject: Bug in FILE-POSITION on UTF-8-encoded files"
 ;;; by Lutz Euler on 2006-03-05 for more details.
-(with-test (:name (:file-position :utf-8)
-                  :fails-on :sbcl)
+(with-test (:name (:file-position :utf-8))
   (let ((path "external-format-test.txt"))
     (with-open-file (s path
                        :direction :output
